import re
from decimal import Decimal

from recipemd.data import Recipe, Ingredient, RecipeParser, Amount


def extract(url,soup):
	if not 'seriouseats.com' in url:
		return

	# title
	title = soup.find('h1',attrs={'class':'title recipe-title'}).text.strip()

	# summary
	summary=''

	summaryPars=soup.find('div',attrs={'class':'recipe-introduction-body'}).find_all('p')

	for par in summaryPars:
		if not 'caption' in par.attrs.get('class',[]):
			summary = summary + par.text + '\n\n'
	summary=summary.strip()

	# servings
	yields = []

	servings = soup.find('span',attrs={'class':'info yield'}).text
	servings_factor = re.compile("\d+").findall(servings)
	if servings_factor:
		yields.append(Amount(Decimal(servings_factor[0]), 'servings'))

	# tags
	tags=[]
	for tag in soup.find_all('a',attrs={'class':'tag'}):
		tags.append(tag.text)

	# ingredients
	ingredients=[]

	for ingred in soup.find_all('li',attrs={'class':'ingredient'}):
		ingredients.append(Ingredient(name=ingred.text))

	# instructions
	instructions=''

	for step in soup.find_all('li',attrs={'class':'recipe-procedure'}):
		stepNumber = step.find('div',attrs={'class':'recipe-procedure-number'}).text.strip()
		stepInstr = step.find('div',attrs={'class':'recipe-procedure-text'}).text.strip()

		instructions = instructions + stepNumber + ' ' + stepInstr + '\n'
	instructions=instructions.strip()

	return Recipe(title=title,ingredients=ingredients,instructions=instructions,description=summary,tags=tags,yields=yields)



import scrape_schema_recipe

from recipemd.data import Recipe, Ingredient, RecipeParser


def extract(url, _):
	try:
		json_recipes = scrape_schema_recipe.scrape_url(url, python_objects=True)
	except:
		return None

	if len(json_recipes) == 0:
		return None
	json_recipe = json_recipes[0]

	tags = []
	if "cookingMethod" in json_recipe:
		tags.append(json_recipe["cookingMethod"])
	if "recipeCategory" in json_recipe:
		append_or_extend(tags, json_recipe["recipeCategory"])
	if "recipeCuisine" in json_recipe:
		tags.append(json_recipe["recipeCuisine"])
	if "keywords" in json_recipe:
		kw = json_recipe["keywords"]
		if isinstance(kw, str):
			kw = kw.split(',')
		append_or_extend(tags, kw)

	description_parts = []
	if "description" in json_recipe:
		description_parts.append(json_recipe["description"])
	if "image" in json_recipe:
		if isinstance(json_recipe["image"], list):
			description_parts.append(f'<img src="{json_recipe["image"][0]}" />')
		else:
			description_parts.append(f'<img src="{json_recipe["image"]}" />')

	yields = []
	if "recipeYield" in json_recipe:
		yields.append(RecipeParser.parse_amount(json_recipe["recipeYield"]))

	recipe = Recipe(
		title=json_recipe["name"],
		description="\n\n".join(description_parts),
		tags=tags,
		yields=yields,
		ingredients=[Ingredient(name=ingred) for ingred in json_recipe["recipeIngredient"]],
		instructions=f'{create_instructions(json_recipe["recipeInstructions"])}\n\n{json_recipe["url"]}',
	)

	return recipe


def create_instructions(instructions, level=2):
	if isinstance(instructions, list):
		return "\n\n".join(create_instructions(el, level) for el in instructions)

	if "@type" in instructions:
		instruction_type = instructions["@type"]

		if instruction_type == "HowToSection":
			child_instructions_list = []
			for child_instruction in instructions["itemListElement"]:
				child_instructions_list.append(create_instructions(child_instruction, level=level+1))
			child_instructions_list = "\n\n".join(child_instructions_list)

			return f'{"#"*level} \n\n{child_instructions_list}'

		if instruction_type == "HowToStep":
			return instructions["text"]

	return str(instructions)


def append_or_extend(list_to_add_to, to_add):
	if isinstance(to_add, list):
		list_to_add_to.extend(to_add)
	else:
		list_to_add_to.append(to_add)


from dataclasses import replace

import requests
from bs4 import BeautifulSoup
from recipemd.data import RecipeParser,Recipe,Ingredient,IngredientGroup


def getJson(url,recipeId):
	# remove queries, fragments
	url=url.split('?')[0]
	url=url.split('#')[0]

	if(url.lower().startswith('http://') or url.lower().startswith('https://')):
		splitted=url.split('/')
		url = splitted[0] + '//' + splitted[2]
	
	r = requests.get(url + '/wp-json/wp/v2/wprm_recipe/' + recipeId)

	if(r.ok):
		return r.json()
	else:
		return None
	
def getText(html):
	return BeautifulSoup(html,'html5lib').text.strip()

def extract(url,soup):
	recipe_id_element = soup.find(attrs={'data-recipe-id':True,'class':'wprm-recipe-container'})

	if not recipe_id_element:
		return

	recipe_id = recipe_id_element.attrs['data-recipe-id']

	data=getJson(url,recipe_id)

	try:
		# title
		title = getText(data['recipe']['name'])
		# summary
		summary = getText(data['recipe']['summary'])
		# servings and tags
		servingsAmount = RecipeParser.parse_amount(data['recipe']['servings'])
		servingsUnit = data['recipe']['servings_unit']
		if servingsUnit != "":
			servingsAmount = replace(servingsAmount, unit=servingsUnit)
		yields = [servingsAmount]

		tags = []
		for tagGroup in data['recipe']['tags'].values():
			for tag in tagGroup:
				tags.append(tag['name'])
		# ingredients
		ingredients=[]

		for ingredGroup in data['recipe']['ingredients']:
			children = []
			if 'name' in ingredGroup:
				title = getText(ingredGroup['name'])
			else:
				title = None
			for ingred in ingredGroup['ingredients']:
				amount = RecipeParser.parse_amount(ingred['amount'])
				unit = ingred['unit'].strip()
				if unit != '':
					amount = replace(amount, unit=unit)
				name = getText('{} {}'.format(ingred['name'],ingred['notes']))
				children.append(Ingredient(name,amount))
			group = IngredientGroup(title=title, ingredients=children)
			ingredients.append(group)
		# instructions
		instructions = ''

		for instrGroup in data['recipe']['instructions']:
			if 'name' in instrGroup:
				instructions = instructions + '## ' + getText(instrGroup['name']) + '\n'
			for index,instr in enumerate(instrGroup['instructions']):
				instructions = instructions + '{}. {}\n'.format(index + 1,getText(instr['text']))

		if 'notes' in data['recipe']:
			instructions = instructions + '\n## Recipe Notes\n\n' + getText(data['recipe']['notes'])

		return Recipe(title=title, ingredients=ingredients, instructions=instructions, description=summary, tags=tags, yields=yields)
	except Exception as e:
		print('failed to extract json:',e)
	# if the json extraction fails, try to extract data from website

	# title
	title = soup.find(attrs={'class': 'wprm-recipe-name'}).text.strip()
	# summary
	summary = soup.find('div',attrs={'class':'wprm-recipe-summary'}).text.strip()

	# yields
	yields = []
	servings = soup.find('span',attrs={'class':'wprm-recipe-details wprm-recipe-servings'})
	if servings:
		servingsAmount = RecipeParser.parse_amount(servings.text.strip())
		servingsUnit = soup.find('span', attrs={'class':'wprm-recipe-details-unit wprm-recipe-servings-unit'}).text.strip()
		if servingsUnit != "":
			servingsAmount = replace(servingsAmount, unit=servingsUnit)
		yields.append(servingsAmount)

	# tags
	tags=[]
	courseTags=soup.find('span',attrs={'class':'wprm-recipe-course'})
	if courseTags:
		courseTags=courseTags.text.split(',')
	else:
		courseTags=[]
	cuisineTags=soup.find('span',attrs={'class':'wprm-recipe-cuisine'})
	if cuisineTags:
		cuisineTags=cuisineTags.text.split(',')
	else:
		cuisineTags=[]
	keywords = soup.find('span',attrs={'class':'wprm-recipe-keyword'})
	if keywords:
		keywords=keywords.text.split(',')
	else:
		keywords=[]
	for tag in courseTags + cuisineTags + keywords:
		tags.append(tag.strip())
	
	# ingredients
	ingreds=[]
	ingredGroups = soup.find_all('div', attrs={'class':'wprm-recipe-ingredient-group'})
	for ingredGroup in ingredGroups:
		groupName=ingredGroup.find('h4', attrs={'class':'wprm-recipe-group-name wprm-recipe-ingredient-group-name'})
		if(groupName):
			title = groupName.text.strip()
		else:
			title = None
		groupIngreds=ingredGroup.find_all('li', attrs={'class':'wprm-recipe-ingredient'})
		children = []
		for ingred in groupIngreds:
			amount = ingred.find('span',attrs={'class':'wprm-recipe-ingredient-amount'})
			if amount:
				amount = RecipeParser.parse_amount(amount.text)
			else:
				amount=None
			unit=ingred.find('span',attrs={'class':'wprm-recipe-ingredient-unit'})
			if unit:
				amount=replace(amount, unit=unit.text)
			name=ingred.find('span',attrs={'class':'wprm-recipe-ingredient-name'})
			if name:
				name=name.text.strip()
			else:
				name=''
			notes=ingred.find('span',attrs={'class':'wprm-recipe-ingredient-notes'})
			if notes:
				notes=notes.text.strip()
			else:
				notes=''
			children.append(Ingredient('{} {}'.format(name,notes).strip(), amount=amount))
		group = IngredientGroup(title=title, ingredients=children)
		ingreds.append(group)

	# instructions
	instructions=''
	instructGroups=soup.find_all('div',attrs={'class':'wprm-recipe-instruction-group'})
	for ingredGroup in instructGroups:
		groupName=ingredGroup.find('h4',attrs={'class':'wprm-recipe-group-name wprm-recipe-instruction-group-name'})

		if groupName:
			instructions = instructions + '## ' + groupName.text.strip() + '\n'
		
		groupInstructs= ingredGroup.find_all('li', attrs={'class':'wprm-recipe-instruction'})
		for index,inst in enumerate(groupInstructs):
			instructions = instructions + str(index+1) + '. ' + inst.text.strip() +'\n'
	# notes
	notesContainer = soup.find('div',attrs={'class':'wprm-recipe-notes-container'})
	if notesContainer:
		notesTitle = notesContainer.find(attrs={'class':'wprm-recipe-header'}).text.strip()
		instructions = instructions + '\n## ' + notesTitle
		for p in notesContainer.find_all('p'):
			instructions= instructions + '\n\n' + p.text.strip()


	return Recipe(title=title, ingredients=ingreds, instructions=instructions, description=summary, tags=tags, yields=yields)



from recipe_scrapers import scrape_me, WebsiteNotImplementedError
from recipemd.data import Recipe, RecipeParser, Ingredient


def extract(url, _):
	try:
		scraper = scrape_me(url)
	except WebsiteNotImplementedError:
		return None

	try:
		description = f'<img src="{scraper.image()}" />'
	except NotImplementedError:
		description = ''

	recipe = Recipe(
		title=scraper.title(),
		description=description,
		yields=[RecipeParser.parse_amount(scraper.yields())],
		ingredients=[Ingredient(name=ingred) for ingred in scraper.ingredients()],
		instructions=scraper.instructions(),
	)

	return recipe



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
import requests
import codecs
import sys
import argparse
from argparse import RawTextHelpFormatter
from recipemd.data import RecipeSerializer,Recipe,Ingredient
import importlib
import os


def extract(url, debug=False):
	try:
		page = requests.get(url)
	except Exception:
		print('No valid URL')
		sys.exit(1)
	soup = BeautifulSoup(page.text, "html5lib")

	pluginFilelist=os.listdir(os.path.dirname(os.path.realpath(__file__))+'/plugins')

	for pluginFile in pluginFilelist:
		if(pluginFile.endswith('.py') and pluginFile != "__init__.py"):
			pluginName='.'+pluginFile[:-3]

			try:
				plugin=importlib.import_module(pluginName,'recipemd_extract.plugins')
				recipe=plugin.extract(url,soup)
				if isinstance(recipe,Recipe):
					return recipe
			except Exception as e:
				if debug:
					raise e
				print('In plugin "',pluginName,'": Error parsing recipe:',e)

def writeRecipe(recipe, filename=None):
	if not filename:
		joinedTitle = '_'.join(recipe.title.lower().split())
		filename = ''.join(c for c in joinedTitle if (c.isalnum() or c in '._')) + '.md'
	with codecs.open(filename, 'w', encoding="utf-8") as f:
		f.write(RecipeSerializer().serialize(recipe))


def main():
	parser = argparse.ArgumentParser(formatter_class=RawTextHelpFormatter)
	parser.add_argument('url', help='URL of the recipe')
	parser.add_argument('filename', help='the file to write to',nargs='?',default=None)
	parser.add_argument('--debug',action='store_true', help='enables debug mode')
	args = parser.parse_args()
	url = args.url
	filename = args.filename

	recipe=extract(url,args.debug)

	if(recipe):
		writeRecipe(recipe, filename)
	else:
		print ('Could not extract recipe')


if __name__ == "__main__":
	main()



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="recipemd-extract",
	version="1.0.0",
	author='AberDerBart',
	author_email='nonatz@web.de',
	description="Extracts recipes from websites and saves them in the RecipeMD format",
	long_description=long_description,
	long_description_content_type="text/markdown",
	packages=setuptools.find_packages(),
	python_requires='>=3.7,<4',
	install_requires=[
		'recipemd~=4.0.3',
		'beautifulsoup4~=4.7.1',
		'requests~=2.22.0',
		'html5lib==1.0.1',
		'scrape-schema-recipe==0.0.4',
		'recipe-scrapers==5.3.0',
	],
	entry_points={
		'console_scripts': [
			'recipemd-extract=recipemd_extract.main:main',
		],
	},
	classifiers=[
		"Development Status :: 4 - Beta",
		"Environment :: Console",
		"Intended Audience :: End Users/Desktop",
		"Programming Language :: Python :: 3",
		"Topic :: Utilities",
		"Topic :: Text Processing :: Markup",
		"Operating System :: OS Independent",
	],
	project_urls={
		'Source': 'https://github.com/AberDerBart/recipemd-extract',
	},
)



from decimal import Decimal

from recipemd.data import RecipeParser, Recipe, Ingredient, Amount


def extract(url,soup):
	if not 'chefkoch.de' in url:
		return

	# title
	title = soup.find('h1', attrs={'class': 'page-title'}).text
	if title == 'Fehler: Seite nicht gefunden' or title == 'Fehler: Rezept nicht gefunden':
		raise ValueError('No recipe found, check URL')
	# summary
	summaryTag = soup.find('div', attrs={'class': 'summary'})
	summary = summaryTag.text if summaryTag else None
	# servings
	servings= soup.find('input', attrs={'id':'divisor'}).attrs['value']
	yields=[Amount(Decimal(servings), f'Portion{"en" if int(servings) > 1 else ""}')]

	# tags
	tags=[]
	tagcloud=soup.find('ul', attrs={'class':'tagcloud'})
	for tag in tagcloud.find_all('a'):
		tags.append(tag.text)
	# ingredients
	table = soup.find('table', attrs={'class': 'incredients'})
	rows = table.find_all('tr')

	ingreds=[]
	for row in rows:
		cols = row.find_all('td')
		cols = [s.text.strip() for s in cols]
		amount = RecipeParser.parse_amount(cols[0])
		ingreds.append(Ingredient(name=cols[1],amount=amount))
	# instructions
	instruct = soup.find('div', attrs={'id': 'rezept-zubereitung'}).text  # only get text
	instruct = instruct.strip()  # remove leadin and ending whitespace
	# write to file
	return Recipe(title=title, ingredients=ingreds, instructions=instruct, description=summary, tags=tags, yields=yields)
